"""Information about QBirthday"""

from PyQt5 import QtCore


__title__ = "QBirthday"
__summary__ = QtCore.QT_TR_NOOP("Birthday reminder")
__uri__ = "https://github.com/lafrech/qbirthday"

__version__ = "0.7.0b3"

__author__ = "Jérôme Lafréchoux"
__email__ = "jerome@jolimont.fr"

__authors__ = (
    "Andreas Bresser <andreas@phidev.org>",
    "Stefan Jurco <stefan.jurco@gmail.com>",
    "Alex Mallo <dernalis@gmail.com>",
    "Thomas Spura <tomspur@fedoraproject.org>",
    "Robert Wildburger <r.wildburger@gmx.at>",
    "Jérôme Lafréchoux <jerome@jolimont.fr>",
)
__artists__ = "Alex Mallo <dernalis@gmail.com>"

__copyright__ = (
    "2015 Jérôme Lafréchoux",
    "2009 Thomas Spura",
    "2009 Andreas Bresser",
    "2007 Alex Mallo",
)

__license__ = "GNU General Public License Version 2"
__license_long__ = """Licensed under the GNU General Public License Version 2

QBirthday is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

QBirthday is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA."""
